@@ -1,11 +1,11 @@
 /*
  * lookup.c --- ext2fs directory lookup operations
- * 
+ *
  * Copyright (C) 1993, 1994, 1994, 1995 Theodore Ts'o.
  *
  * %Begin-Header%
- * This file may be redistributed under the terms of the GNU Public
- * License.
+ * This file may be redistributed under the terms of the GNU Library
+ * General Public License, version 2.
  * %End-Header%
  */
 
@@ -23,15 +23,15 @@ struct lookup_struct  {
 	int		len;
 	ext2_ino_t	*inode;
 	int		found;
-};	
+};
 
 #ifdef __TURBOC__
  #pragma argsused
 #endif
 static int lookup_proc(struct ext2_dir_entry *dirent,
-		       int	offset,
-		       int	blocksize,
-		       char	*buf,
+		       int	offset EXT2FS_ATTR((unused)),
+		       int	blocksize EXT2FS_ATTR((unused)),
+		       char	*buf EXT2FS_ATTR((unused)),
 		       void	*priv_data)
 {
 	struct lookup_struct *ls = (struct lookup_struct *) priv_data;
