@@ -159,7 +159,7 @@ int main (int argc, char ** argv)
 	int		print_min_size = 0;
 	int		fd, ret;
 	blk_t		new_size = 0;
-	blk_t		max_size = 0;
+	blk64_t		max_size = 0;
 	blk_t		min_size = 0;
 	io_manager	io_ptr;
 	char		*new_size_str = 0;
@@ -250,10 +250,8 @@ int main (int argc, char ** argv)
 				device_name);
 			exit(1);
 		}
-		if (!(mount_flags & EXT2_MF_MOUNTED) || (mtpt[len-1] == 0)) {
-			free(mtpt);
+		if (!(mount_flags & EXT2_MF_MOUNTED) || (mtpt[len-1] == 0))
 			break;
-		}
 		free(mtpt);
 		len = 2 * len;
 	}
@@ -347,6 +345,13 @@ int main (int argc, char ** argv)
 	min_size = calculate_minimum_resize_size(fs);
 
 	if (print_min_size) {
+		if (!force && ((fs->super->s_state & EXT2_ERROR_FS) ||
+			       ((fs->super->s_state & EXT2_VALID_FS) == 0))) {
+			fprintf(stderr,
+				_("Please run 'e2fsck -f %s' first.\n\n"),
+				device_name);
+			exit(1);
+		}
 		printf(_("Estimated minimum size of the filesystem: %u\n"),
 		       min_size);
 		exit(0);
@@ -369,7 +374,7 @@ int main (int argc, char ** argv)
 	 * defaults and for making sure the new filesystem doesn't
 	 * exceed the partition size.
 	 */
-	retval = ext2fs_get_device_size(device_name, fs->blocksize,
+	retval = ext2fs_get_device_size2(device_name, fs->blocksize,
 					&max_size);
 	if (retval) {
 		com_err(program_name, retval,
@@ -377,7 +382,7 @@ int main (int argc, char ** argv)
 		exit(1);
 	}
 	if (force_min_size)
-		new_size = calculate_minimum_resize_size(fs);
+		new_size = min_size;
 	else if (new_size_str) {
 		new_size = parse_num_blocks(new_size_str,
 					    fs->super->s_log_block_size);
@@ -387,6 +392,14 @@ int main (int argc, char ** argv)
 			exit(1);
 		}
 	} else {
+		/* Take down devices exactly 16T to 2^32-1 blocks */
+		if (max_size == (1ULL << 32))
+			max_size--;
+		else if (max_size > (1ULL << 32)) {
+			com_err(program_name, 0, _("New size too large to be "
+				"expressed in 32 bits\n"));
+			exit(1);
+		}
 		new_size = max_size;
 		/* Round down to an even multiple of a pagesize */
 		if (sys_page_size > fs->blocksize)
@@ -426,7 +439,7 @@ int main (int argc, char ** argv)
 	}
 	if (!force && (new_size > max_size)) {
 		fprintf(stderr, _("The containing partition (or device)"
-			" is only %u (%dk) blocks.\nYou requested a new size"
+			" is only %llu (%dk) blocks.\nYou requested a new size"
 			" of %u blocks.\n\n"), max_size,
 			fs->blocksize / 1024, new_size);
 		exit(1);
@@ -447,16 +460,22 @@ int main (int argc, char ** argv)
 				device_name);
 			exit(1);
 		}
-	printf("Resizing the filesystem on %s to %u (%dk) blocks.\n",
+		printf(_("Resizing the filesystem on "
+			 "%s to %u (%dk) blocks.\n"),
 		       device_name, new_size, fs->blocksize / 1024);
 		retval = resize_fs(fs, &new_size, flags,
 				   ((flags & RESIZE_PERCENT_COMPLETE) ?
 				    resize_progress_func : 0));
 	}
+	free(mtpt);
 	if (retval) {
 		com_err(program_name, retval, _("while trying to resize %s"),
 			device_name);
-		ext2fs_close (fs);
+		fprintf(stderr,
+			_("Please run 'e2fsck -fy %s' to fix the filesystem\n"
+			  "after the aborted resize operation.\n"),
+			device_name);
+		ext2fs_close(fs);
 		exit(1);
 	}
 	printf(_("The filesystem on %s is now %u blocks long.\n\n"),
